#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Graphes - énoncé
# 
# Ce notebook introduit [matplotlib](http://matplotlib.org/) et d'autres modules Python qui permettent de tracer des graphes et bâtis sur la même logique que matplotlib.

# Pour avoir des graphiques inclus dans le notebook, il faut ajouter cette ligne et l'exécuter en premier.

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')


# On change le style pour un style plus moderne, celui de [ggplot](http://ggplot2.org/) :

# In[2]:


import matplotlib.pyplot as plt
plt.style.use('ggplot')


# In[3]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Données
# 
# Pour tous les exemples qui suivent, on utilise les résultat [élection présidentielle de 2012](https://www.data.gouv.fr/fr/datasets/election-presidentielle-2012-resultats-572124/). Si vous n'avez pas le module [actuariat_python](http://www.xavierdupre.fr/app/actuariat_python/helpsphinx/index.html), il vous suffit de recopier le code de la fonction [elections_presidentielles](http://www.xavierdupre.fr/app/actuariat_python/helpsphinx/_modules/actuariat_python/data/elections.html#elections_presidentielles) qui utilise la fonction [read_excel](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.read_excel.html).
# La fonction utilise des données stockées localement afin que le code ci-dessous fonctionne toujours même si le format des données change sur le site *data.gouv.fr*.

# In[4]:


from actuariat_python.data import elections_presidentielles
dict_df = elections_presidentielles(local=True, agg="dep")


# In[5]:


list(dict_df.keys())


# In[6]:


dict_df["dep1"].head()


# In[7]:


dict_df["dep2"].head()


# On corrige le code du département ``01`` --> ``1``.

# In[8]:


def cleandep(s):
    if isinstance(s, str):
         r = s.lstrip('0')
    else:
        r = str(s)
    return r
dict_df["dep1"]["Code du département"] = dict_df["dep1"]["Code du département"].apply(cleandep)
dict_df["dep2"]["Code du département"] = dict_df["dep2"]["Code du département"].apply(cleandep)


# In[9]:


deps = dict_df["dep1"].merge(dict_df["dep2"], 
                                       on="Code du département",
                                       suffixes=("T1", "T2"))
deps.columns


# In[10]:


deps["rHollandeT1"] = deps['François HOLLANDE (PS)T1'] / (deps["VotantsT1"] - deps["Blancs et nulsT1"])
deps["rSarkozyT1"] = deps['Nicolas SARKOZY (UMP)T1'] / (deps["VotantsT1"] - deps["Blancs et nulsT1"])
deps["rNulT1"] = deps["Blancs et nulsT1"] / deps["VotantsT1"]
deps["rHollandeT2"] = deps["François HOLLANDE (PS)T2"] / (deps["VotantsT2"] - deps["Blancs et nulsT2"])
deps["rSarkozyT2"] = deps['Nicolas SARKOZY (UMP)T2'] / (deps["VotantsT2"] - deps["Blancs et nulsT2"])
deps["rNulT2"] = deps["Blancs et nulsT2"] / deps["VotantsT2"]
data = deps[["Code du département", "Libellé du départementT1", 
             "VotantsT1", "rHollandeT1", "rSarkozyT1", "rNulT1",
             "VotantsT2", "rHollandeT2", "rSarkozyT2", "rNulT2"]]
data_elections = data # parfois data est remplacé dans la suite
data.head()


# In[11]:


deps.to_excel("deps.xlsx")
dict_df["dep1"].to_excel("T1.xlsx")
dict_df["dep2"].to_excel("T2.xlsx")


# ## De pandas à matplotlib
# 
# Lorsqu'on construit un graphique avec des données stockées dans un [DataFrame](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.html), on suit généralement le processus suivant :
# 
# * Voir si un graphique correspond dans la page [visualisation](http://pandas.pydata.org/pandas-docs/stable/visualization.html) de pandas
# * Voir la [gallerie](http://matplotlib.org/gallery.html) de [matplotlib](http://matplotlib.org/index.html)
# * Chercher un exemple de graphique sur un moteur de recherche pour tomber sur une page comme celle-ci [Using Python libraries to plot two horizontal bar charts sharing same y axis](http://stackoverflow.com/questions/27694221/using-python-libraries-to-plot-two-horizontal-bar-charts-sharing-same-y-axis)
# * Assembler différentes sources

# ### plot
# 
# La méthode [plot](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.plot.html?highlight=plot#pandas.DataFrame.plot) permet de faire la plupart des graphiques standards (voir [Plotting](http://pandas.pydata.org/pandas-docs/stable/visualization.html)).

# In[12]:


data.plot(x="Libellé du départementT1", y=["rHollandeT2", "rSarkozyT2"], figsize=(16,5));


# In[13]:


data.plot(x="Libellé du départementT1", y=["rHollandeT2", "rSarkozyT2"],
          figsize=(16,5), kind="bar", stacked=True);


# In[14]:


data.plot(x="rHollandeT1", y="rHollandeT2", figsize=(16,5), 
          kind="scatter", label="s1", title="correlation");


# ### superposition
# 
# La méthode [plot](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.plot.html?highlight=plot#pandas.DataFrame.plot) retourne un objet de type [Axes](http://matplotlib.org/api/axes_api.html). On peut superposer plusieurs courbes sur le même graphique en s'assurant que la seconde courbe utilise le même objet.

# In[15]:


ax = data.plot(x="rHollandeT1", y="rHollandeT2", figsize=(16,5), 
               kind="scatter", label="H", title="correlation")
data.plot(x="rSarkozyT1", y="rSarkozyT2", kind="scatter", label="S", ax=ax, c="red");


# On ajoute une ligne avec la méthode [Axes.plot](http://matplotlib.org/api/axes_api.html?highlight=plot#matplotlib.axes.Axes.plot) ou du text avec [text](http://matplotlib.org/api/text_api.html#matplotlib.text.Text) :

# In[16]:


ax = data.plot(x="rHollandeT1", y="rHollandeT2", figsize=(16,5), 
               kind="scatter", label="H", title="correlation")
data.plot(x="rSarkozyT1", y="rSarkozyT2", kind="scatter", label="S", ax=ax, c="red")
ax.plot([0.2,0.7], [0.2,0.7], "g--")
ax.text(0.5, 0.5, "rien au dessous", weight="bold", rotation="-30");


# ### plusieurs graphes sur la même figure
# 
# *pandas* crée une [Figure](http://matplotlib.org/api/pyplot_api.html?highlight=figure#matplotlib.pyplot.figure) de façon implicite avec un seul graphe. Pour créer plusieurs graphes, il faut créer ce type d'objet en précisant qu'il y aura plusieurs [Axes](http://matplotlib.org/api/axes_api.html) avec la fonction [subplots](http://matplotlib.org/api/pyplot_api.html?highlight=figure#matplotlib.pyplot.subplots) et les transmettre à *pandas*. On peut également partager l'axe des X ou l'axe des Y.

# In[17]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1, 2, figsize=(16,5), sharey=True)
data.plot(x="rHollandeT1", y="rHollandeT2", figsize=(16,5), 
          kind="scatter", label="H", ax=axes[0])
data.plot(x="rSarkozyT1", y="rSarkozyT2", kind="scatter", 
          label="S", ax=axes[1], c="red")
axes[0].plot([0.2,0.7], [0.2,0.7], "g--")
axes[1].plot([0.2,0.7], [0.2,0.7], "g--");


# ### matplotlib sans pandas
# 
# On peut se passer de pandas et s'inspirer d'un graphe de la gallerie pour ajouter des points dépendants du nombre de votants [scatter_demo](http://matplotlib.org/examples/shapes_and_collections/scatter_demo.html) et ajouter une légende manuellement avec la méthode [legend](http://matplotlib.org/api/axes_api.html?highlight=legend#matplotlib.axes.Axes.legend).

# In[18]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1, 1, figsize=(16,5))
c = axes.scatter(x=data["rHollandeT1"], 
            y=data["rHollandeT2"], 
            s=data["VotantsT1"]/5000, alpha=0.5)
axes.plot([0.2,0.7], [0.2,0.7], "g--")
axes.legend( (c,), ("H",) );


# ## Pandas et graphes prêts à l'emploi

# ### histogrammes
# 
# avec [hist](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.hist.html?highlight=hist#pandas.DataFrame.hist)

# In[19]:


data.hist(figsize=(8, 8));


# Le paramètre *figsize* permettrait de modifier la taille du graphique.

# ### correlations
# 
# avec [scatter_matrix](http://pandas.pydata.org/pandas-docs/stable/visualization.html#scatter-matrix-plot)

# In[20]:


from pandas.plotting import scatter_matrix
scatter_matrix(data, alpha=0.2, figsize=(14, 14), diagonal='kde')
print("-");


# ## cartes avec cartopy
# 
# Les coordonnées sur une carte se font avec des [coordonnées géographiques](https://fr.wikipedia.org/wiki/Coordonn%C3%A9es_g%C3%A9ographiques) : longitude et latitude. La distance entre deux lieux géographiques se calcule grâce à la [distance de Haversine](https://en.wikipedia.org/wiki/Haversine_formula). Les graphes se font avec [cartopy](http://scitools.org.uk/cartopy/).

# ### une carte simple
# 
# On la choisit centrée sur la France. La carte se dessine avec [matplotlib](https://matplotlib.org/) auquel [cartopy](https://scitools.org.uk/cartopy/docs/latest/) ajoute un système de projection différent. Comme elle accepte un argument ``ax``, il est possible de changer sa taille ou de la juxtaposer à côté d'un autre graphe. Le module [cartopy](https://scitools.org.uk/cartopy/docs/latest/) ne contient pas toutes les informations sur le territoire français et certaines [options](https://scitools.org.uk/cartopy/docs/latest/matplotlib/feature_interface.html#cartopy.feature.GSHHSFeature) ne semble pas avoir d'effet.

# In[21]:


import cartopy.crs as ccrs
import cartopy.feature as cfeature
import matplotlib.pyplot as plt

fig = plt.figure(figsize=(7,7))
ax = fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())
ax.set_extent([-5, 10, 42, 52])

ax.add_feature(cfeature.OCEAN.with_scale('50m'))
# ax.add_feature(cfeature.COASTLINE)
# ax.add_feature(cfeature.RIVERS)  # pas d'effet, cartopy ne connaît pas les rivières en France
# ax.add_feature(cfeature.BORDERS, linestyle=':')
# cette instruction télécharge un fichier (10m=14Mo, 50m, 110m)
# il faut la résolution 10m pour la France
ax.add_feature(cfeature.STATES.with_scale('10m')) 
ax.set_title('France');


# La méthode [with_scale](https://scitools.org.uk/cartopy/docs/latest/matplotlib/feature_interface.html?highlight=with_scale#cartopy.feature.NaturalEarthFeature.with_scale) propose trois résolution 10m, 50m, 110m. La module [cartopy](https://scitools.org.uk/cartopy/docs/latest/) n'inclut que la résolution 110m, le reste doit être téléchargé.

# ### exercice 1 : centrer la carte de la France

# In[22]:





# ### ajouter du texte ou une marque
# 
# Sur une carte, on veut la plupart du temps ajouter du texte. On reprend le début de ce code qu'on place dans une fonction, puis on place Paris. On utilise pour cela les fonctions standard de *matplotlib* mais on convertit les coordonnées géographiques en coordonnées relatives au graphe (donc dans un repère différent).

# In[23]:


import cartopy.crs as ccrs
import cartopy.feature as cfeature
import matplotlib.pyplot as plt

def carte_france(figsize=(7, 7)):
    fig = plt.figure(figsize=figsize)
    ax = fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())
    ax.set_extent([-5, 10, 42, 52])

    ax.add_feature(cfeature.OCEAN.with_scale('50m'))
    ax.add_feature(cfeature.RIVERS.with_scale('50m'))
    ax.add_feature(cfeature.BORDERS.with_scale('50m'), linestyle=':')
    ax.set_title('France');
    return ax

carte_france();


# In[24]:


import matplotlib.pyplot as plt

ax = carte_france()

lon = 2.3488000
lat = 48.853410
ax.plot([lon], [lat], 'ro', markersize=6)
ax.text(lon, lat, "Paris");


# On connaît rarement les coordonnées de chaque ville mais un moteur de recherche donne rapidement des pistes pour trouver ces données. Il faut néanmoins s'assurer que le licence autorise ce qu'on l'intention de faire avec :
# 
# [Liste des villes de France en SQL, CSV ou XML](http://sql.sh/736-base-donnees-villes-francaises)

# In[25]:


from pyensae.datasource import download_data
download_data("villes_france.csv", url="http://sql.sh/ressources/sql-villes-france/")


# In[26]:


cols = ["ncommune", "numero_dep", "slug", "nom", "nom_simple", "nom_reel", 
        "nom_soundex", "nom_metaphone", "code_postal",
        "numero_commune", "code_commune", "arrondissement", "canton", 
        "pop2010", "pop1999", "pop2012",
        "densite2010", "surface", "superficie", "dlong", "dlat", "glong", 
        "glat", "slong", "slat", "alt_min", "alt_max"]
import pandas
df = pandas.read_csv("villes_france.csv", header=None,low_memory=False, names=cols)


# In[27]:


df.head()


# ### exercice 2 : placer les plus grandes villes de France sur la carte

# In[28]:





# ### départements
# 
# Pour dessiner des formes sur une carte, il faut connaître les coordonnées de ces formes. L'article suivant [Matplotlib Basemap tutorial 10: Shapefiles Unleached, continued](http://www.geophysique.be/2013/02/12/matplotlib-basemap-tutorial-10-shapefiles-unleached-continued/) permet de dessiner les départements belges. On va s'en inspirer pour dessiner les départements français. La première chose à faire est de récupérer des données géographiques. Une façon simple de les trouver est d'utiliser un moteur de recherche avec le mot clé **shapefile** inclus dedans : c'est le format du fichier. *shapefile france* permet d'obtenir quelques sources. En voici d'autres :
# 
# * [GADM](http://www.gadm.org/) : database of Global Administrative Areas
# * [OpenData.gouv commune](https://www.data.gouv.fr/fr/datasets/geofla-communes/) : base de données sur data.gouv.fr
# * [The National Map Small-Scale Collection](http://nationalmap.gov/small_scale/#chpbound) : Etats-Unis
# * [ArcGIS](https://developers.arcgis.com/javascript/jsapi/esri.basemaps-amd.html) : API Javascripts
# * [Natural Earth](http://www.naturalearthdata.com/) : Natural Earth is a public domain map dataset available at 1:10m, 1:50m, and 1:110 million scales. Featuring tightly integrated vector and raster data, with Natural Earth you can make a variety of visually pleasing, well-crafted maps with cartography or GIS software.
# * [thematicmapping](http://thematicmapping.org/downloads/world_borders.php) : World Borders Dataset
# * [OpenStreetMap Data Extracts](http://download.geofabrik.de/) : OpenStreetMap data
# * [OpenStreetMapData](http://openstreetmapdata.com/) : OpenStreetMap data
# * [Shapefile sur Wikipedia](http://wiki.openstreetmap.org/wiki/Shapefiles) : contient divers liens vers des sources de données
# 
# La première chose à vérifier est la licence associées aux données : on ne peut pas en faire ce qu'on veut. Pour cet exemple, j'ai choisi la première source de données, GADM. La licence n'est pas précisée explicitement (on peut trouver *happy to share* sur le site, la page wikipedia [GADM](https://en.wikipedia.org/wiki/GADM) précise : *GADM is not freely available for commercial use. The GADM project created the spatial data for many countries from spatial databases provided by national governments, NGO, and/or from maps and lists of names available on the Internet (e.g. from Wikipedia).* C'est le choix que j'avais fait en 2015 mais l'accès à ces bases a probablement changé car l'accès est restreint. J'ai donc opté pour les bases accessibles depuis [data.gouv.fr](https://www.data.gouv.fr/fr/datasets/geofla-departements-30383060/). Leur seul inconvénient est que les coordonnées sont exprimées dans une projection de type [Lambert 93](https://fr.wikipedia.org/wiki/Projection_conique_conforme_de_Lambert). Cela nécessite une conversion.

# In[29]:


from pyensae.datasource import download_data
try:
    download_data("GEOFLA_2-1_DEPARTEMENT_SHP_LAMB93_FXX_2015-12-01.7z",
              website="https://wxs-telechargement.ign.fr/oikr5jryiph0iwhw36053ptm/telechargement/inspire/" + \
              "GEOFLA_THEME-DEPARTEMENTS_2015_2$GEOFLA_2-1_DEPARTEMENT_SHP_LAMB93_FXX_2015-12-01/file/")
except Exception as e:
    # au cas le site n'est pas accessible
    download_data("GEOFLA_2-1_DEPARTEMENT_SHP_LAMB93_FXX_2015-12-01.7z", website="xd")


# In[30]:


from pyquickhelper.filehelper import un7zip_files
try:
    un7zip_files("GEOFLA_2-1_DEPARTEMENT_SHP_LAMB93_FXX_2015-12-01.7z", where_to="shapefiles")
    departements = 'shapefiles/GEOFLA_2-1_DEPARTEMENT_SHP_LAMB93_FXX_2015-12-01/GEOFLA/1_DONNEES_LIVRAISON_2015/' + \
                   'GEOFLA_2-1_SHP_LAMB93_FR-ED152/DEPARTEMENT/DEPARTEMENT.shp'
except FileNotFoundError as e:
    # Il est possible que cette instruction ne fonctionne pas.
    # Dans ce cas, on prendra une copie de ce fichier.
    import warnings
    warnings.warn("Plan B parce que " + str(e))
    download_data("DEPARTEMENT.zip")
    departements = "DEPARTEMENT.shp"
    
import os
if not os.path.exists(departements):
    raise FileNotFoundError("Impossible de trouver '{0}'".format(departements))


# La license accompagne les données : *ce produit est téléchargeable et utilisable gratuitement sous licence [Etalab](https://www.etalab.gouv.fr/licence-ouverte-open-licence)*. Pour un usage commercial, il faut faire attentation à la license associée aux données. Le seul inconvénient des données *GEOFLA* est que certaines sont données dans le système de coordonnées [Lambert 93](https://fr.wikipedia.org/wiki/Projection_conique_conforme_de_Lambert) (voir aussi [Cartographie
# avec R](https://www.sylvaindurand.org/cartographie-avec-R/)).

# In[31]:


shp = departements
import shapefile
r = shapefile.Reader(shp)
shapes = r.shapes()
records = r.records()
len(shapes), len(records)


# In[32]:


r.bbox


# On regarde une zone en particulier mais on réduit la quantité de données affichées :

# In[33]:


d = shapes[0].__dict__.copy()
d["points"] = d["points"][:10]
d


# 350 départements, sûr ?

# In[34]:


records[0], records[1]


# In[35]:


len(set([r[6] for r in records]))


# Puis je récupère le code final (toujours à [Matplotlib Basemap tutorial 10: Shapefiles Unleached, continued](http://www.geophysique.be/2013/02/12/matplotlib-basemap-tutorial-10-shapefiles-unleached-continued/)) en l'adaptant pour la France. Petite astuce, on utilie la fonction [lambert932WGPS](https://github.com/sdpython/ensae_teaching_cs/blob/master/src/ensae_teaching_cs/helpers/geo_helper.py) du module [ensae_teaching_cs](http://www.xavierdupre.fr/app/ensae_teaching_cs/). On recopie le code ici :

# In[36]:


import math


def lambert932WGPS(lambertE, lambertN):

    class constantes:
        GRS80E = 0.081819191042816
        LONG_0 = 3
        XS = 700000
        YS = 12655612.0499
        n = 0.7256077650532670
        C = 11754255.4261

    delX = lambertE - constantes.XS
    delY = lambertN - constantes.YS
    gamma = math.atan(-delX / delY)
    R = math.sqrt(delX * delX + delY * delY)
    latiso = math.log(constantes.C / R) / constantes.n
    sinPhiit0 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * math.sin(1)))
    sinPhiit1 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * sinPhiit0))
    sinPhiit2 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * sinPhiit1))
    sinPhiit3 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * sinPhiit2))
    sinPhiit4 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * sinPhiit3))
    sinPhiit5 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * sinPhiit4))
    sinPhiit6 = math.tanh(latiso + constantes.GRS80E * math.atanh(constantes.GRS80E * sinPhiit5))

    longRad = math.asin(sinPhiit6)
    latRad = gamma / constantes.n + constantes.LONG_0 / 180 * math.pi

    longitude = latRad / math.pi * 180
    latitude = longRad / math.pi * 180

    return longitude, latitude

lambert932WGPS(99217.1, 6049646.300000001), lambert932WGPS(1242417.2, 7110480.100000001)


# In[37]:


import cartopy.crs as ccrs
import matplotlib.pyplot as plt
ax = carte_france((8,8))

from matplotlib.collections import LineCollection
import shapefile
import geopandas
from shapely.geometry import Polygon
from shapely.ops import cascaded_union, unary_union
 
shp = departements
r = shapefile.Reader(shp)
shapes = r.shapes()
records = r.records()

polys = []
for i, (record, shape) in enumerate(zip(records, shapes)):
    # les coordonnées sont en Lambert 93
    if i == 0:
        print(record, shape.parts)
    geo_points = [lambert932WGPS(x,y) for x, y in shape.points]
    if len(shape.parts) == 1:
        # Un seul polygone
        poly = Polygon(geo_points)
    else:
        # Il faut les fusionner.
        ind = list(shape.parts) + [len(shape.points)]
        pols = [Polygon(geo_points[ind[i]:ind[i+1]]) for i in range(0, len(shape.parts))]
        try:
            poly = unary_union(pols)
        except Exception as e:
            print("Cannot merge: ", record)
            print([_.length for _ in pols], ind)
            poly = Polygon(geo_points)
    polys.append(poly)
    
data = geopandas.GeoDataFrame(geometry=polys)
# cmap -> voir https://matplotlib.org/users/colormaps.html
data.plot(ax=ax, cmap='tab20', edgecolor='black');
# Ou pour définir des couleurs spécifiques.
# geopandas.plotting.plot_polygon_collection(ax, data['geometry'], data['colors'], values=None)


# ### exercice 3 : résultats des élections par départements
# 
# Ce n'est pas toujours évident !

# In[38]:





# ## seaborn
# 
# [seaborn](http://stanford.edu/~mwaskom/software/seaborn/) propose des graphiques inéressants pour un statisticien. La [gallerie](http://stanford.edu/~mwaskom/software/seaborn/examples/index.html) en donne un bon aperçu. On retravaille peu les graphiques. Le code suivant montre les corrélations entre variables [pairplot](http://stanford.edu/~mwaskom/software/seaborn/examples/scatterplot_matrix.html).
# 
# **warning** les warnings sont la plupart du temps dûs au fait que *seaborn* a été testé sur une version antérieure d'une de ses dépendances comme *matplotlib* et qu'il n'est pas encore à jour pour tenir compte des derniers développement.

# In[39]:


import seaborn
seaborn.pairplot(data_elections);


# Celui-ci est aussi intéressant : [clustermap](http://stanford.edu/~mwaskom/software/seaborn/examples/structured_heatmap.html) pour étudier les corrélations.

# In[40]:


import seaborn
seaborn.set(font="monospace")

cmap = seaborn.diverging_palette(h_neg=210, h_pos=350, s=90, l=30, as_cmap=True)

seaborn.clustermap(data_elections.corr(), linewidths=.5, figsize=(13, 13), cmap=cmap);


# ## bokeh
# 
# [bokeh](http://bokeh.pydata.org/en/latest/) propose des graphiques en javascript. La [gallerie](http://bokeh.pydata.org/en/latest/docs/gallery.html) est moins fournie que celle de matplotib. Le principale avantage de bokeh est de proposer gaphiques zoomables (interactifs).

# ### initialisation
# 
# La première étape est de signifier que la sortie se fera dans un notebook.

# In[41]:


from bokeh.plotting import output_notebook
output_notebook()


# ### premier graphe

# On utilise bokeh pour un simple graphique XY. on peut choisir les différentes options interactives, zoom ... Voir [tools](http://bokeh.pydata.org/en/latest/docs/user_guide/tools.html#specifying-tools).

# In[42]:


from bokeh.plotting import figure, show

p = figure(title = "élections")
p.title.text = "élections"
p.circle(data_elections["rHollandeT1"], data_elections["rHollandeT2"], color="red", fill_alpha=0.2, size=10, legend="H")
p.circle(data_elections["rSarkozyT1"], data_elections["rSarkozyT2"], color="blue", fill_alpha=0.2, size=10, legend="S")
p.line([0.2,0.7], [0.2,0.7], line_color="green", line_dash="dashed")
p.xaxis.axis_label = "tour 1"
p.yaxis.axis_label = "tour 2"
show(p)


# ### ajouter du texte
# 
# Les différents éléments qu'on peut ajouter au graphe s'appelle des [Glyphes](http://bokeh.pydata.org/en/latest/docs/reference/bokehjs.html). La documentation manque parfois de précision. Pour la fonction [text](http://bokeh.pydata.org/en/latest/docs/reference/bokehjs.html#text), il faut préciser le texte à afficher sous forme de liste.

# In[43]:


from bokeh.plotting import figure, show

p = figure(title = "élections")
p.title.text = "élections"
p.circle(data_elections["rHollandeT1"], data_elections["rHollandeT2"], color="red", fill_alpha=0.2, size=10, legend="H")
p.circle(data_elections["rSarkozyT1"], data_elections["rSarkozyT2"], color="blue", fill_alpha=0.2, size=10, legend="S")
p.line([0.2,0.7], [0.2,0.7], line_color="green", line_dash="dashed")
p.xaxis.axis_label = "tour 1"
p.yaxis.axis_label = "tour 2"


def display_text(p, row):
    p.text(x=row["rHollandeT1"], y=row["rHollandeT2"], text=[row["Libellé du départementT1"]],
          text_font_size="8pt", color="black", text_align="left", text_baseline="middle", angle=0)
    return row["Libellé du départementT1"]

data_elections.apply(lambda row: display_text(p, row), axis=1)

show(p)


# ### composition de graphes
# 
# Voir [linked brushing](http://bokeh.pydata.org/en/latest/docs/user_guide/interaction.html#linked-brushing).

# In[44]:


from bokeh.plotting import figure, show, gridplot
size = 400

ph = figure(title = "élections", width=size, height=size)
ph.title.text = "élections"
ph.circle(data_elections["rHollandeT1"], data_elections["rHollandeT2"], color="red", fill_alpha=0.2, size=10, legend="H")
ph.line([0.2,0.7], [0.2,0.7], line_color="green", line_dash="dashed")
ph.xaxis.axis_label = "tour 1"
ph.yaxis.axis_label = "tour 2"

ps = figure(title = "élections", width=size, height=size)
ps.title.text = "élections"
ps.circle(data_elections["rSarkozyT1"], data_elections["rSarkozyT2"], color="blue", fill_alpha=0.2, size=10, legend="S")
ps.line([0.2,0.7], [0.2,0.7], line_color="green", line_dash="dashed")
ps.xaxis.axis_label = "tour 1"
ps.yaxis.axis_label = "tour 2"

p = gridplot([[ph, ps]])

show(p)


# ### interactions avec bokeh
# 
# Tout est expliqué dans la page [interaction](http://bokeh.pydata.org/en/latest/docs/user_guide/interaction.html).
# 

# In[45]:


from bokeh.models.widgets import Panel, Tabs
from bokeh.io import show
from bokeh.plotting import figure

ph = figure(title="élections")
ph.circle(data_elections["rHollandeT1"], data_elections["rHollandeT2"], color="red", fill_alpha=0.2, size=10, legend="H")
ph.line([0.2,0.7], [0.2,0.7], line_color="green", line_dash="dashed")
ph.xaxis.axis_label = "tour 1"
ph.yaxis.axis_label = "tour 2"

ps = figure(title = "élections")
ps.title.text = "élections"
ps.circle(data_elections["rSarkozyT1"], data_elections["rSarkozyT2"], color="blue", fill_alpha=0.2, size=10, legend="S")
ps.line([0.2,0.7], [0.2,0.7], line_color="green", line_dash="dashed")
ps.xaxis.axis_label = "tour 1"
ps.yaxis.axis_label = "tour 2"

tab1 = Panel(child=ph, title="Hollande")
tab2 = Panel(child=ps, title="Sarkozy")

tabs = Tabs(tabs=[ tab1, tab2 ])

show(tabs)


# Si on connaît un peu de javascript, on peut créer des graphes qui peuvent interagir avec la souris à n'importe quel endroit du graphe.

# ## interaction avec matplotlib (ou bokeh)

# ### dropdown

# Le module [interact](https://github.com/ipython/ipywidgets) permet de changer le contenu d'une cellule en fonction d'un bouton, d'une barre de défilement... Le code de cette fonction est sur [github](https://github.com/ipython/ipywidgets/blob/master/ipywidgets/widgets/interaction.py). La liste des widget possible est dans le notebook [WidgetList](https://github.com/ipython/ipywidgets/blob/477cb8046e3217b134762f53c66816c45d688a20/examples/Widget%20List.ipynb)

# In[46]:


import matplotlib.pyplot as plt
from IPython.html.widgets import interact, Dropdown

def plot(cand):
    
    fig, axes = plt.subplots(1, 1, figsize=(14,5), sharey=True)
    if cand=="Hollande":
        data_elections.plot(x="rHollandeT1", y="rHollandeT2", kind="scatter", label="H", ax=axes)
    else:
        data_elections.plot(x="rSarkozyT1", y="rSarkozyT2", kind="scatter", label="S", ax=axes, c="red")
    axes.plot([0.2,0.7], [0.2,0.7], "g--")
    return axes

cand = Dropdown(options=['Hollande', 'Sarkozy'], value='Hollande', description='candidat')

interact(plot, cand=cand)
print("")


# Ces interactions ne sont pas limitées à matplotlib. Tout type de sortie peut dépendre d'un *widget*.

# ### exercice 4 : même code, widget différent 

# In[47]:





# ## autres options
# 
# Il existe un grande nombre de modules permettant de dessiner. Les plus récents utilisent le javascript.
# 
# * static
# 
#     * [ggplot](http://ggplot.yhathq.com/) : aspect similare à matplotlib version ggplot mais la syntaxe est différente
# 
# * Javascript
# 
#     * [pygal](http://www.pygal.org/en/latest/) : voir exemple plus bas, aspect réussi, le module prévoit une extension pour les [carte](http://www.pygal.org/en/latest/documentation/types/maps/pygal_maps_fr.html), le résultat nécessite l'ajout de quelques scripts Javascript
#     * [mpld3](http://mpld3.github.io/) : créer un graphe avec la syntaxe Javascript et le transformer en Javascript
#     * [folium](https://github.com/python-visualization/folium) : Javascript + [OpenStreetMap](https://www.openstreetmap.org/)
# 

# In[48]:


html_pygal = """
<script type="text/javascript" src="http://kozea.github.com/pygal.js/javascripts/svg.jquery.js"></script>
<script type="text/javascript" src="http://kozea.github.com/pygal.js/javascripts/pygal-tooltips.js"></script>
{pygal_render}
"""

from IPython.core.display import HTML
import pygal
xy_chart = pygal.XY(stroke=False)
xy_chart.title = 'Elections 2012'
xy = list(zip(data_elections["rHollandeT1"], data_elections["rHollandeT2"]))
xy_chart.add('H', xy)
HTML(html_pygal.format(pygal_render=xy_chart.render().decode()))


# In[49]: